#include "project.h"
#include "stdio.h"
#include "stdlib.h"
#include "tty_utils.h"
#include "func_list.h"
#include "i2c_utils.h"
#include "pcf8563.h"

void do_help(void) ;

void do_get(void) ;
void do_set(void) ;
void get_date(void) ;
void get_time(void) ;
void do_bcd_test(void) ;
void do_run(void) ;
void do_stop(void) ;
void do_dump(void) ;
void do_init(void) ;

f_list_type main_func_list[] = {
    { "help", do_help,     "show help message" },
    { "get",  do_get,      "get {all | date | time }" },
    { "set",  do_set,      "set {all | date | time }" },
    { "date", get_date,    "get date (YYYY/MM/DD)" },
    { "time", get_time,    "get time {hh:mm:ss}" },
    { "run",  do_run,      "run RTC clock"},
    { "stop", do_stop,     "stop RTC clock"},
    { "dump", do_dump,     "dump PCF8563 registers"}, 
    { "init", do_init,     "initialize values with YYYY/MM/DD hh:mm:ss"},
    { "bcd",  do_bcd_test, "test bcd function" },
    { 0,      0,           0 } /* list terminator */
} ;

int year    = 2021 ;
int month   = 12 ;
int day     = 3 ;
int hour    = 12 ;
int minute  = 34 ;
int second  = 56 ;
int century = 2000 ;
int weekday = 5 ; /* Friday */

void init_hardware(void)
{
    CyGlobalIntEnable; /* Enable global interrupts. */
    I2C_Start() ;
    tty_init() ;
}

int main(void)
{
    func_ptr func ;
    
    init_hardware() ;
    
    splash("PCF8563 Test") ;
    
    do_help() ;

    prompt("> ") ;
    for(;;)
    {
        if (get_line()) {
            func = get_func(main_func_list, str) ;
            if (func) {
                func() ;
            } else {
                print("Unknown Command: ") ;
                print(str) ;
                print("\r\n") ;
                do_help() ;
            }
            prompt("> ") ;
        }
    }
}

void do_help(void)
{
    show_help(main_func_list) ;
}

void do_run(void)
{
    pcf8563_run() ;
}

void do_stop(void)
{
    pcf8563_stop() ;
}

void do_dump(void)
{
    char    buf[16] ;
    uint8_t regs[16] ;
    int i ;
    pcf8563_get_regs(regs) ;
    for (i = 0 ; i < 0x10 ; i++) {
        snprintf(buf, 16, "%02X ", regs[i]) ;
        print(buf) ;
    }
    print("\n\r") ;
}

void do_init(void)
{
    char    buf[32], date_str[32], time_str[32] ;
    int     c ;
    uint8_t regs[16] ;
    int i ;
    
    sscanf(str, "%*s %s %s",
        date_str, time_str) ;
    sscanf(date_str, "%d/%d/%d",
        &year, &month, &day) ;
    
    sscanf(time_str, "%d:%d:%d",
        &hour, &minute, &second) ;
    
    print("Setting registers for ") ;
    snprintf(buf, 32, "%d/%d/%d ",
        year, month, day) ;
    print(buf) ;
    snprintf(buf, 32, "%02d:%02d:%02d\n\r",
        hour, minute, second) ;
    print(buf) ;
    if (year >= 2000) {
        c = 0 ;
    } else {
        c = 0x80 ;
    }
    year %= 100 ;
    regs[0]  = 0x00 ;
    regs[1]  = 0x00 ;
    regs[2]  = byte2bcd(second) ;
    regs[3]  = byte2bcd(minute) ;
    regs[4]  = byte2bcd(hour) ;
    regs[5]  = byte2bcd(day) ;
    regs[6]  = byte2bcd(weekday) ;
    regs[7]  = byte2bcd(month) | c ;
    regs[8]  = byte2bcd(year) ;
    regs[9]  = 0x00 ;
    regs[10] = 0x00 ;
    regs[11] = 0x00 ;
    regs[12] = 0x00 ;
    regs[13] = 0x00 ;
    regs[14] = 0x00 ;
    regs[15] = 0x00 ; 
    for (i = 0 ; i < 0x10 ; i++) {
        snprintf(buf, 16, "%02X ", regs[i]) ;
        print(buf) ;
    }
    print("\n\r") ;
    pcf8563_set_regs(regs) ;
}

void do_bcd_test(void)
{
    int value ;
    uint8_t bcd ;
    sscanf(str, "%*s %d", &value) ;
    bcd = byte2bcd(value & 0xFF) ;
    snprintf(str, STR_BUF_LEN, "(input) %d = %02X (%d) in BCD\n\r",
        value, bcd, bcd) ;
    print(str) ;
    value = bcd2byte(bcd) ;
    snprintf(str, STR_BUF_LEN, "(bcd) %02X (%d) = %d\n\r",
        bcd, bcd, value) ;
    print(str) ;    
}

void get_date(void) 
{
    int  c ;
    char buf[32] ;
    c     = pcf8563_get_century() ;
    year  = pcf8563_get_years() ;
    month = pcf8563_get_months() ;
    day   = pcf8563_get_days() ;

    if (c == 0) {
        century = 2000 ;
    } else {
        century = 1900 ;
    }
    year += century ;
    snprintf(buf, 30, "%d/%d/%d\n\r",
        year, month, day) ;
    print(buf) ;
}

void get_time(void) 
{
    char buf[32] ;
    hour   = pcf8563_get_hours() ;
    minute = pcf8563_get_minutes() ;
    second = pcf8563_get_seconds() ;

    snprintf(buf, 30, "%02d:%02d:%02d\n\r",
        hour, minute, second) ;
    print(buf) ;
}

void set_year(void)
{
    print("Year: ") ;
    while(get_line() == 0) { } ;
    sscanf(str, "%d", &year) ;
    if (year >= 2000) {
        century = 2000 ;
        pcf8563_set_century(0) ;
    } else {
        century = 1900 ;
        pcf8563_set_century(1) ;
    }
    year %= 100 ;
    pcf8563_set_years(year) ;
}

void set_month(void)
{
    print("Month(1-12): ") ;
    while(get_line() == 0) {} ;
    sscanf(str, "%d", &month) ;
    if (month < 1) {
        month = 1 ;
    } else if (month > 12) {
        month = 12 ;
    }
    pcf8563_set_months(month) ;
}

void set_day(void)
{
    print("Day: ") ;
    while(get_line() == 0) {} ;
    sscanf(str, "%d", &day) ;
    if (day < 1) {
        day = 1 ;
    } else if (day > 31) {
        day = 31 ;
    }
    pcf8563_set_days(day) ;
}

void set_date(void)
{
    set_year() ;
    set_month() ;
    set_day() ;    
}

void set_hour(void) 
{
    print("Hour(0-23): ") ;
    while(get_line() == 0) {} ;
    sscanf(str, "%d", &hour) ;
    if (hour < 0) {
        hour = 0 ;
    } else if (hour > 23) {
        hour = hour % 24 ;
    }
    pcf8563_set_hours(hour) ;
}

void set_minute(void)
{
    print("Minute (0-59): ") ;
    while(get_line() == 0) {} ;
    sscanf(str, "%d", &minute) ;
    if (minute < 0) {
        minute = 0 ;
    } else if (minute > 59) {
        minute = minute % 60 ;
    }
    pcf8563_set_minutes(minute) ;
}

void set_second(void)
{
    print("Second (0-59): ") ;
    while(get_line() == 0) {} ;
    sscanf(str, "%d", &second) ;
    if (second < 0) {
        second = 0 ;
    } else if (second > 59) {
        second = second % 60 ;
    }
    pcf8563_set_seconds(second) ;
}

void set_time(void)
{
    set_hour() ;
    set_minute() ;
    set_second() ;
}

void do_get(void) 
{
    char buf[32] ;
    sscanf(str, "%*s %s", buf) ;
    switch(*buf) {
    case 'A': // all
    case 'a':
        get_date() ;
        get_time() ;
        break ;
    case 'D': // date
    case 'd':
        get_date() ;
        break ;
    case 'T': // time
    case 't':
        get_time() ;
        break ;
    default:
        print("Unknown Command: ") ;
        print(str) ;
        print("\n\r") ;
        do_help() ;
        break ;
    }
}

void do_set(void)
{
    char buf[32] ;
    sscanf(str, "%*s %s", buf) ;
    switch(*buf) {
    case 'A': // all
    case 'a':
        set_date() ;
        set_time() ;
        break ;
    case 'D': // date
    case 'd':
        set_date() ;
        break ;
    case 'T': // time
    case 't':
        set_time() ;
        break ;
    default:
        print("Unknown Command: ") ;
        print(str) ;
        print("\n\r") ;
        do_help() ;
        break ;
    }   
}

/* [] END OF FILE */
