#include "project.h"
#include "stdio.h"
#include "tty_utils.h"
#include "i2c_utils.h"
#include "pcf8563.h"

#define PCF8563_I2C_ADDRESS 0x51

/**
 * Register Addresses
 */
#define REG_CONTROL_STATUS_1 0x00
#define REG_CONTROL_STATUS_2 0x01
#define REG_VL_SECONDS       0x02
#define REG_MINUTES          0x03
#define REG_HOURS            0x04
#define REG_DAYS             0x05
#define REG_WEEKDAYS         0x06
#define REG_CENTURY_MONTHS   0x07
#define REG_YEARS            0x08
#define REG_MINUTE_ALARM     0x09
#define REG_HOUR_ALARM       0x0A
#define REG_DAY_ALARM        0x0B
#define REG_WEEKDAY_ALARM    0x0C
#define REG_CLKOUT_CONTROL   0x0D
#define REG_TIMER_CONTROL    0x0E
#define REG_TIMER            0x0F

/**
 * REG_CONTROL_STATUS_1
 * Bit 7: TEST1
 *        0: Normal mode 
 *        1: EXT_CLK test mode
 * Bit 6: 0     : default value is logic 0
 * Bit 5: STOP
 *        0: RTC source clock runs 
 *        1: all RTC divider chaain flip-flops are asynchronously set to logic 0:
 *           the RTC clock is stopped (CLKOUT at 32.768 kHz is still available)
 * Bit 4: 0     : default value is logic 0
 * Bit 3: TESTC
 *        0: Power-on reset override facility is disabled;
 *           set to logic 0 for nomarl operation
 *        1: Power-on reset override may be enabled
 * Bit [2:0] : 0 : default value is logic 0
 */

/**
 * REG_CONTROL_STATUS_2
 * Bit[7:5]: 0 : default value is logic 0
 * Bit 4   : TI_TP
 *           0: /INT is active when TF is active (subject to the status of TIE)
 *           1: /INT pulses active according to Table6 (subject to the status of TIE)
 *              note that if AF and AIE are active then /INT will be permanently active
 * Bit 3   : AF
 *           0 (read)  : alarm flag inactive
 *           1 (read)  : alarm flag active
 *           0 (write) : alarm flag is cleared
 *           1 (write) : alarm flag remains unchanged
 * Bit 2   : TF
 *           0 (read)  : timer flag inactive
 *           1 (read)  : timer flag active
 *           0 (write) : timer flag is cleared
 *           1 (write) : timer flag remains unchanged
 * Bit 1   : AIE
 *           0: alarm interrupt disabled
 *           1: alarm interrupt enabled
 * Bit 0   : TIE
 *           0: timer interrupt disabled
 *           1: timer interrupt enabled
 */

/**
 * Table6
 * /INT operation (bit TI_TP = 1)
 *
 * Source clock (Hz)    /INT period (s) (*1)
 *                      n = 1 (*2)        n > 1 (*2)
 * 4096                 1/8192            1/4096
 *   64                 1/128             1/64
 *    1                 1/64              1/64
 *    1/60              1/64              1/64
 *
 * (*1) TF and /INT become active simultaneously.
 * (*2) n = loaded countdown value. Timer stopped when n = 0.
 */

/**
 * Time and date registers
 * Table7 REG_VL_SECONDS - Validity and Seconds register (address 0x02) bit description
 * Bit 7     : VL
 *             0: clock integrity is guranteed
 *             1: integrity of the clock information is no longer guaranteed
 * Bit [6:0] : SECONDS[6:0] 
 *             00 to 59 : the current seconds, coded in BCD format.
 *                        Example: seconds register contains x101 1001 = 59 seconds
 * 
 * Table8 REG_MINUTES - Minutes register (address 0x03) bit description
 * Bit 7     : X   not relevant
 * Bit [6:0] : MINUTES[6:0] 00 to 59 the current minutes, coded in BCD format
 *
 * Table9 REG_HOURS - Hours register (address 0x04) bit description
 * Bit [7:6] : X   not relevant
 * Bit [5:0] : HOURS[5:0] 00 to 23 the current hours, coded in BCD format
 *
 * Table10 REG_DAYS - Days register (address 0x05) bit description
 * Bit [7:0] : X   not relevant
 * Bit [5:0] : DAYS[5:0] 01 to 31 the current day, coded in BCD format
 *             The PCF8563 compensates for leap years by adding a 29th day to Feburary
 *             if the year counter contains a value which is exactly divisble by 4, 
 *             including the year 00.
 *
 * Table11 REG_WEEKDAYS - Weekdays register (address 0x06) bit description
 * Bit [7:3] : X   not relevant
 * Bit [2:0] : WEEKDAYS[2:0] 0 to 6 the current weekday, coded in BCD format.
 *             These bits may be re-assigned by the user.
 *
 * Weekday   Bit7 Bit6 Bit5 Bit4 Bit3 Bit2 Bit1 Bit0
 * Sunday    x    x    x    x    x    0    0    0
 * Monday    x    x    x    x    x    0    0    1
 * Tuesday   x    x    x    x    x    0    1    0
 * Wednesday x    x    x    x    x    0    1    1
 * Thursday  x    x    x    x    x    1    0    0
 * Friday    x    x    x    x    x    1    0    1
 * Saturday  x    x    x    x    x    1    1    0
 *
 * Table13 REG_CENTURY_MONTHS - Century and Months register (address 0x07) bit description
 * Bit 7     : C   century; this bit is toggled when the year register overflows from 99 to 00
 *             0   indicates the century is 20xx
 *             1   indicates the century is 19xx
 * Bit [6:5] : X   not relevant
 * Bit [4:0] : MONTHS[4:0] 01 to 12 the current month, coded in BCD format.
 *
 * Month     Bit7 Bit6 Bit5 Bit4 Bit3 Bit2 Bit1 Bit0
 * January   C    x    x    0    0    0    0    1
 * February  C    x    x    0    0    0    1    0
 * March     C    x    x    0    0    0    1    1
 * April     C    x    x    0    0    1    0    0
 * May       C    x    x    0    0    1    0    1
 * June      C    x    x    0    0    1    1    0
 * July      C    x    x    0    0    1    1    1
 * August    C    x    x    0    1    0    0    0
 * September C    x    x    0    1    0    0    1
 * October   C    x    x    1    0    0    0    0
 * November  C    x    x    1    0    0    0    1
 * December  C    x    x    1    0    0    1    0
 *
 * Table15 REG_YEARS - Years register (address 0x08) bit description
 * Bit [7:0] : YEARS[7:0] 00 to 99 the current year, coded in BCD format
 */

uint8_t byte2bcd(uint8_t in)
{
    uint8_t bcd ;
    bcd = ((in / 10) << 4) | (in %10) ;
    return( bcd ) ;
}

uint8_t bcd2byte(uint8_t in)
{
    uint8_t value = 0 ;
    value = (((in & 0xF0) >> 4) * 10) + (in & 0x0F) ;
    return( value ) ;
}

void    pcf8563_init(void) 
{
}

void    pcf8563_dump_regs(void) 
{
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    
}

uint8_t pcf8563_get_seconds(void) 
{
    uint8_t value = 0 ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_VL_SECONDS, &value) ;
    value &= 0x7F ; /* clear VL bit */
    value = bcd2byte(value) ;
    return( value ) ;
}

uint8_t pcf8563_get_minutes(void) 
{
    uint8_t value = 0 ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_MINUTES, &value) ;
    value &= 0x7F ; /* clear MSB bit */
    value = bcd2byte(value) ;
    return( value ) ;    
}

uint8_t pcf8563_get_hours(void) 
{
    uint8_t value = 0 ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_HOURS, &value) ;
    value &= 0x3F ; /* clear Bit[7:6] */
    value = bcd2byte(value) ;
    return( value ) ;   
}

uint8_t pcf8563_get_days(void)
{
    uint8_t value = 0 ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_DAYS, &value) ;
    value &= 0x3F ; /* clear Bit[7:3] */
    value = bcd2byte(value) ;
    return( value ) ;     
}

uint8_t pcf8563_get_weekdays(void) 
{
    uint8_t value = 0 ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_WEEKDAYS, &value) ;
    value &= 0x3F ; /* clear Bit[7:6] */
    value = bcd2byte(value) ;
    return( value ) ;     
}

uint8_t pcf8563_get_months(void) 
{
    uint8_t value = 0 ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_CENTURY_MONTHS, &value) ;
    value &= 0x1F ; /* clear Bit[7:5] */
    value = bcd2byte(value) ;
    return( value ) ;
}

uint8_t pcf8563_get_years(void)
{
    uint8_t value = 0 ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_YEARS, &value) ;
    value = bcd2byte(value) ;
    return( value ) ;
}

uint8_t pcf8563_get_century(void) 
{
    uint8_t value = 0 ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_CENTURY_MONTHS, &value) ;
    value >>= 7 ; /* C is Bit[7] */
    return( value ) ;
}

uint8_t pcf8563_get_VL(void) 
{
    uint8_t value = 0 ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_VL_SECONDS, &value) ;
    value >>= 7 ; /* VL is Bit[7] */
    return( value ) ;    
}

void    pcf8563_set_seconds(uint8_t seconds) 
{
    uint8_t value ;
    seconds = byte2bcd(seconds) ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_VL_SECONDS, &value) ;
    value = (value & 0x80) | (seconds | 0x7F) ;
    i2c_write_reg(REG_VL_SECONDS, value) ;
}

void    pcf8563_set_minutes(uint8_t minutes) 
{
    uint8_t value ;
    minutes = byte2bcd(minutes) ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_MINUTES, &value) ;
    value = (minutes & 0x7F) ; /* clear Bit[7] */
    i2c_write_reg(REG_MINUTES, value) ;
}

void    pcf8563_set_hours(uint8_t hours) 
{
    uint8_t value ;
    hours = byte2bcd(hours) ;    
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_HOURS, &value) ;
    value = (hours & 0x3F) ; /* clear Bit[7:6] */
    i2c_write_reg(REG_HOURS, value) ;
}

void    pcf8563_set_days(uint8_t days) 
{
    uint8_t value ;
    days = byte2bcd(days) ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_DAYS, &value) ;
    value = (days & 0x3F) ; /* clear Bit[7:6] */
    i2c_write_reg(REG_DAYS, value) ;
}

void    pcf8563_set_weekdays(uint8_t weekdays) 
{
    uint8_t value ;
    weekdays = byte2bcd(weekdays) ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_WEEKDAYS, &value) ;
    value = (weekdays & 0x3F) ; /* clear Bit[7:6] */
    i2c_write_reg(REG_WEEKDAYS, value) ;
}

void    pcf8563_set_months(uint8_t months) 
{
    uint8_t value, century ;
    months = byte2bcd(months) ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_CENTURY_MONTHS, &value) ;
    century = value & 0x80 ;
    value = (months & 0x1F) | century ;
    i2c_write_reg(REG_CENTURY_MONTHS, value) ;    
}

void    pcf8563_set_years(uint8_t years) 
{
    uint8_t value ;
    years = byte2bcd(years) ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_YEARS, &value) ;
    value = years ; /* clear Bit[7:6] */
    i2c_write_reg(REG_YEARS, value) ;  
}

void    pcf8563_set_century(uint8_t century) 
{
    uint8_t value ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_CENTURY_MONTHS, &value) ;
    value = (value & 0x7F) | (century << 7) ;
    i2c_write_reg(REG_CENTURY_MONTHS, value) ; 
}

void    pcf8563_set_VL(uint8_t VL) 
{
    uint8_t value ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_VL_SECONDS, &value) ;
    value = (value & 0x7F) | (VL << 7) ;
    i2c_write_reg(REG_VL_SECONDS, value) ;    
}


void    pcf8563_run(void) 
{
    uint8_t value ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_CONTROL_STATUS_1, &value) ;
    value ^= 0x20 ; /* clear Bit[5] (Stop) */
    i2c_write_reg(REG_CONTROL_STATUS_1, value) ;
}

void    pcf8563_stop(void) 
{
    uint8_t value ;
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_reg(REG_CONTROL_STATUS_1, &value) ;
    value |= 0x20 ; /* set Bit[5] (Stop) */
    i2c_write_reg(REG_CONTROL_STATUS_1, value) ;    
}

void    pcf8563_get_regs(uint8_t regs[]) 
{
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_read_regs_w_buf(0, regs, 16) ;  
}

void    pcf8563_set_regs(uint8_t regs[]) 
{
    i2c_set_slave_address(PCF8563_I2C_ADDRESS) ;
    i2c_write_regs_w_buf(0, regs, 16) ;
}
